@@ -253,12 +253,53 @@ double shrinkage_laplace(double R2, int n, int p, double alpha)
 		- dn*log(1.0 + ghat*(1. -R2))
 		+ log(sigmahat))  +  lognc + log(ghat);
    logden = logBF_hyperGprior_laplace( R2,  n, p, alpha);
+/* lognc is included here so that it cancels wth lognc for denominator
    shrinkage = exp(lognum - logden);
 /*   Rprintf("%f %f %f  %f %f %f \n", ghat, sigmahat, normalpart, lognum, logden, shrinkage);  */
   if (p == 1) shrinkage = 1.0;
   return(shrinkage);
 }
 
+double log_laplace_2F1(double a, double b, double c, double z)
+ {  
+   
+   double  ghat,logmarg,sigmahat, D, A, B, C,root1, root2;
+
+/*  Laplace approximation in terms of exp(tau) = g  */
+//
+//  Integrate[g^(b/2-1) (1 + g)^(a - c)/2 (1 + (1 - z) g)^(-a/2) dg 
+//  Integrate[e^tau b/2 (1 + e^g)^(a - c)/2 *( 1 + (1 -z)e^g)^(-a/2) dtau  
+//
+   A = (b - c)*(1. - z);
+   B = 2.*b - c + (a-b)*z;
+   C = b;
+   D = B*B - 4.*A*C;
+
+   if (D < 0 )    Rprintf("ERROR: In Laplace approximation to 2F1");
+
+   root1 = (-B - sqrt(D))/(2.*A);
+   root2 = (-B + sqrt(D))/(2.*A);
+   
+   Rprintf("+root = %lf root = %lf\n", root2, root1);
+
+   ghat = (2.*b)/(c + b*(z - 2.0) - a*z + sqrt(c*c + 4*a*b*z -  2.0*(a + b)*c *z + (a -b )*(a-b)*z*z)/(2.*(b - c)*(z - 1)));
+ 
+   if (ghat <= 0.0)  {
+     Rprintf("ERROR: In Laplace approximation to  logmarg,  ghat =  %f  z = %lf\n", ghat, z);
+     ghat =  -(c +b*(-2. + z) - a*z + sqrt(c*c + 4*a*b*z -  2.0*(a + b)*c *z + (a -b )*(a-b)*z*z)/(2.*(b - c)*(z - 1.)));
+   }
+     sigmahat =1.0/(.5*(-a + c)*((ghat/(1 + ghat))*(1 - ghat/(1 + ghat))) +
+		    a*((ghat*(1.-z))/(1.+ghat*(1.-z))*
+		       (1. - (ghat*(1.-z))/(1.+ghat*(1.-z)))));
+
+    if (sigmahat <= 0 ) Rprintf("ERROR in LAPLACE APPROXIMATION to in 2F1 sigmhat = %f, ghat =  %f  z = %f \n", sigmahat, ghat, z); 
+    logmarg = .5*( log(2.0*PI) 
+		   - (a - c)*log(1.0 + ghat)
+		   -  a*log(1.0 + ghat*(1. - z)) 
+		   + log(sigmahat)) 
+               + (.5*b - 1.)*log(ghat);
+  return(logmarg);
+ }	
 
 
 double logBF_EB(double R2, int n, int p, double alpha)
@@ -273,61 +314,3 @@ double logBF_EB(double R2, int n, int p, double alpha)
     if (p == 1) logmarg = 0.0;
     return(logmarg);
 }
-
-void compute_modelprobs(SEXP Rmodelprobs,  SEXP Rlogmarg, int k)
-{
-  int m;
-  double nc, bestmarg, *modelprobs, *logmarg;
-
-  logmarg = REAL(Rlogmarg);
-  modelprobs = REAL(Rmodelprobs);
-
-  bestmarg = logmarg[0];
-  nc = 0.0;
-
-  for (m = 0; m < k; m++) {
-    if (logmarg[m] > bestmarg) bestmarg = logmarg[m];
-  }
-
-  for (m = 0; m < k; m++) {
-    modelprobs[m] = logmarg[m] - bestmarg;
-    nc += exp(modelprobs[m]); }	
-
-  for (m = 0; m < k; m++) {
-    modelprobs[m] = exp(modelprobs[m] - log(nc));
- }	
-}
-
-void compute_margprobs(SEXP modelspace, SEXP modeldim, SEXP Rmodelprobs, double *margprobs, int k, int p)
-{
-  int m, j, *model;
-  double *modelprobs;
-
-  modelprobs = REAL(Rmodelprobs);
-
-  for (j=0; j< p; j++)  margprobs[j] = 0.0;
-
-   for(m=0; m< k; m++) {
-     model = INTEGER(VECTOR_ELT(modelspace,m));
-     for (j = 0; j < INTEGER(modeldim)[m]; j ++) {
-       margprobs[model[j]] += modelprobs[m];
-     } }
-
-}
-
-
-
-void compute_margprobs_old(Bit **models, SEXP Rmodelprobs, double *margprobs, int k, int p)
-{
-  int m, j;
-  double *modelprobs;
-  modelprobs = REAL(Rmodelprobs);
-
-  for (j=0; j< p; j++) {
-    margprobs[j] = 0.0;
-   for(m=0; m< k; m++) {
-     if (models[m][j])
-        margprobs[j] += modelprobs[m];
-      }
-    }
-}
